Attribute VB_Name = "MTimerMM"
Option Explicit

Private Declare Function timeSetEvent Lib "winmm.dll" _
        (ByVal uDelay As Long, ByVal uResolution As Long, _
        ByVal lpFunction As Long, ByVal dwUser As Long, _
        ByVal uFlags As Long) As Long

Private Declare Function timeKillEvent Lib "winmm.dll" _
        (ByVal uID As Long) As Long

Private Declare Function timeGetTime Lib "winmm.dll" () As Long

Private Declare Function timeBeginPeriod Lib "winmm.dll" _
        (ByVal uPeriod As Long) As Long

Private Declare Function timeEndPeriod Lib "winmm.dll" _
        (ByVal uPeriod As Long) As Long

Private Declare Function CreateWindowEx Lib "user32" _
        Alias "CreateWindowExA" (ByVal dwExStyle As Long, _
        ByVal lpClassName As String, ByVal lpWindowName As String, _
        ByVal dwStyle As Long, ByVal x As Long, ByVal y As Long, _
        ByVal nWidth As Long, ByVal nHeight As Long, _
        ByVal hWndParent As Long, ByVal hMenu As Long, _
        ByVal hInstance As Long, lpParam As Any) As Long

Private Declare Function DestroyWindow Lib "user32" _
        (ByVal hWnd As Long) As Long

Private Declare Function DefWindowProc Lib "user32" _
        Alias "DefWindowProcA" (ByVal hWnd As Long, _
        ByVal wMsg As Long, ByVal wParam As Long, _
        ByVal lParam As Long) As Long

Private Declare Function CallWindowProc Lib "user32" _
        Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, _
        ByVal hWnd As Long, ByVal Msg As Long, ByVal wParam As Long, _
        ByVal lParam As Long) As Long

Private Declare Function SetWindowLong Lib "user32" _
        Alias "SetWindowLongA" (ByVal hWnd As Long, _
        ByVal nIndex As Long, ByVal dwNewLong As Long) As Long

Private Declare Function GetWindowLong Lib "user32" _
        Alias "GetWindowLongA" (ByVal hWnd As Long, _
        ByVal nIndex As Long) As Long
'
' PostMessage MUST be defined in a type library or
' the multimedia timer callback procedure will GPF'
'
' Private Declare Function PostMessage Lib "user32" _
'        Alias "PostMessageA" (ByVal hWnd As Long, _
'        ByVal wMsg As Long, ByVal wParam As Long, _
'        ByVal lParam As Long) As Long

Private Const SWP_NOZORDER = &H4
Private Const GWL_WNDPROC = -4
Private Const WS_BORDER = &H800000
Private Const WS_CAPTION = &HC00000
Private Const WS_CHILD = &H40000000
Private Const WS_CHILDWINDOW = (WS_CHILD)
Private Const WS_CLIPCHILDREN = &H2000000
Private Const WS_CLIPSIBLINGS = &H4000000
Private Const WS_DISABLED = &H8000000
Private Const WS_DLGFRAME = &H400000
Private Const WS_MINIMIZE = &H20000000
Private Const WS_OVERLAPPED = &H0&
Private Const TIME_CALLBACK_FUNCTION = &H0
Private Const MAX_PRECISION = 1
'
' User-defined messages start at WM_USER.
'
Private Const WM_CLOSE = &H10
Private Const WM_USER As Long = &H400
Private Const WM_TIMER_FIRED As Long = (WM_USER + &H1001)

Private mlOriginalWinProc As Long
Private mobjTimers        As Collection

Public Function fRegister(objTimer As CTimerMM) As Long
Dim objData As CObjData
Dim hWnd    As Long

If mobjTimers Is Nothing Then
    Set mobjTimers = New Collection
End If

hWnd = fCreateWin()

If hWnd <> 0 Then
    Call pStartMessageCapture(hWnd)
Else
    Err.Raise 51, "MTimerMM::fRegister", "Unable to create timer control window"
End If

Set objData = New CObjData
    
With objData
    Set .objTimer = objTimer
    .hWnd = hWnd
End With
'
' Use the window handle as the key so the timer
' callback procedure can identify the timer that fired.
'
mobjTimers.Add objData, CStr(hWnd)

Set objData = Nothing
fRegister = hWnd
End Function

Public Sub pUnregister(lObjectId As Long)
'
' Ignore errors when removing a reference.
'
On Error Resume Next

With mobjTimers(CStr(lObjectId))
    Call pEndMessageCapture(.hWnd)
    Call pDestroyWin(.hWnd)
End With

mobjTimers.Remove CStr(lObjectId)

If mobjTimers.Count = 0 Then
    Set mobjTimers = Nothing
End If
End Sub

Public Function fSetTimer(ByVal lDelay As Long, ByVal lResolution As Long, ByVal evtEventType As EEventTypes, lTimerId As Long) As Long
Dim lFlags As Long
'
' Set the period for maximum precision.
'
Call timeBeginPeriod(MAX_PRECISION)

lFlags = TIME_CALLBACK_FUNCTION Or evtEventType

fSetTimer = timeSetEvent(lDelay, lResolution, AddressOf pTimerProcMM, lTimerId, lFlags)
End Function

Public Sub pKillTimer(ByVal lngTimerId As Long)
'
' Kill the timer.
'
Call timeKillEvent(lngTimerId)
'
' Clear the previously set period.
'
Call timeEndPeriod(MAX_PRECISION)
End Sub

Public Function fGetTime() As Long

fGetTime = timeGetTime()
End Function

Public Sub pTimerProcMM(ByVal uID As Long, ByVal uMsg As Long, _
    ByVal dwUser As Long, ByVal dw1 As Long, ByVal dw2 As Long)
'
' dwUser parameter has the window handle.
'
Call PostMessageA(dwUser, WM_TIMER_FIRED, 0, dwUser)
End Sub

Public Function fCreateWin() As Long
Dim hWnd   As Long
Dim lStyle As Long

fCreateWin = 0

lStyle = (WS_OVERLAPPED Or WS_MINIMIZE)

hWnd = CreateWindowEx(0, "STATIC", "TimerMMWindow", lStyle, 0, 0, 100, 100, 0, 0, 0, 0)

If hWnd <> 0 Then
    fCreateWin = hWnd
Else
    MsgBox "Unable to create window, LastError = " & Err.LastDllError
End If
End Function

Public Sub pDestroyWin(ByVal hWnd As Long)
'
' Close window.
'
Call PostMessageA(hWnd, WM_CLOSE, 0, 0)
End Sub

Public Sub pStartMessageCapture(ByVal hWnd As Long)
Dim lWinProc As Long

mlOriginalWinProc = 0

lWinProc = GetWindowLong(hWnd, GWL_WNDPROC)

If lWinProc <> 0 Then
    Call SetWindowLong(hWnd, GWL_WNDPROC, AddressOf fLocalWinProc)
Else
    Err.Raise 51, "", "Unable to get default window proc handle"
End If

mlOriginalWinProc = lWinProc
End Sub

Public Sub pEndMessageCapture(ByVal hWnd As Long)

Call SetWindowLong(hWnd, GWL_WNDPROC, mlOriginalWinProc)
End Sub

Public Function fLocalWinProc(ByVal hWnd As Long, ByVal uMsg As Long, _
    ByVal wParam As Long, ByVal lParam As Long) As Long

Dim objData  As CObjData
Dim objTimer As CTimerMM

Select Case uMsg
    Case WM_TIMER_FIRED:
        '
        ' lParam has the window handle from the timer
        ' callback procedure for the timer to fire.
        '
        Set objData = mobjTimers(CStr(lParam))
        Set objTimer = objData.objTimer
        objTimer.Fire
        Set objTimer = Nothing
        Set objData = Nothing
        fLocalWinProc = DefWindowProc(hWnd, uMsg, wParam, lParam)
    Case Else:
        fLocalWinProc = CallWindowProc(mlOriginalWinProc, hWnd, uMsg, wParam, lParam)
End Select
End Function
