VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CTimerMM"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Description = "Multimedia callback timer"
Option Explicit

Private Const TIME_ONESHOT = 0
Private Const TIME_PERIODIC = 1
Private Const TIME_CALLBACK_FUNCTION = &H0
Private Const TIMERR_NOERROR = 0
Private Const UNDEFINED = 0

Private mlInterval    As Long
Private mlResolution  As Long
Private mlTimerId     As Long
Private mlStartTime   As Long
Private mlEndTime     As Long
Private mlObjectId    As Long
Private mbRefSet      As Boolean
Private mbEnabled     As Boolean
Private mevtEventType As EEventTypes

Public Enum EEventTypes
    etOneShot = TIME_ONESHOT
    etPeriodic = TIME_PERIODIC
End Enum

Public Event Timer(ByVal lMilliseconds As Long)

Private Sub Class_Initialize()

mbRefSet = False
mbEnabled = False
mlInterval = 0
mevtEventType = etOneShot
mlTimerId = UNDEFINED
mlObjectId = UNDEFINED
End Sub

Private Sub Class_Terminate()

If mlTimerId <> UNDEFINED Then
    Call pKillTimer(mlTimerId)
End If

If mlObjectId <> UNDEFINED Then
    Call pUnregister(mlObjectId)
End If
End Sub

Public Property Get Enabled() As Boolean
Enabled = mbEnabled
End Property

Public Property Let Enabled(ByVal bValue As Boolean)
Dim Delay      As Long
Dim Resolution As Long
Dim UserData   As Long
Dim Flags      As Long

If mbEnabled = bValue Then
    Exit Property
End If

If mlInterval <= 0 Then
    Err.Raise 380, "CTimerMM::Enabled", "Interval property must be positive"
End If

If mlResolution < 0 Then
    Err.Raise 380, "CTimerMM::Enabled", "Resolution property must be non-negative"
End If

mbEnabled = bValue

If mbEnabled Then
    If mlObjectId = UNDEFINED Then
        mlObjectId = fRegister(Me)
    End If
    Delay = mlInterval
    Resolution = mlResolution
    mlTimerId = fSetTimer(Delay, Resolution, mevtEventType, mlObjectId)
    mlStartTime = fGetTime()
Else
    Call pKillTimer(mlTimerId)
    mlTimerId = UNDEFINED
    Call pUnregister(mlObjectId)
    mlObjectId = UNDEFINED
End If
End Property

Public Property Get Interval() As Long
Interval = mlInterval
End Property

Public Property Let Interval(ByVal lValue As Long)

If lValue <= 0 Then
    Err.Raise 380, "CTimerMM::Interval", "Interval property must be positive"
End If

mlInterval = lValue
End Property

Public Property Get Resolution() As Long

Resolution = mlResolution
End Property

Public Property Let Resolution(ByVal lValue As Long)

If lValue < 0 Then
    Err.Raise 380, "CTimerMM::Resolution", "Resolution property must be non-negative"
End If

mlResolution = lValue
End Property

Public Property Get EventType() As EEventTypes

EventType = mevtEventType
End Property

Public Property Let EventType(ByVal eValue As EEventTypes)

mevtEventType = eValue
End Property

Friend Sub Fire()
'
' Fire - "fires" the timer event in the client
'
mlEndTime = fGetTime()

RaiseEvent Timer(mlEndTime - mlStartTime)

mlStartTime = fGetTime()
End Sub
