VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsStopWatch"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Private Declare Function timeGetTime Lib "winmm.dll" () As Long
Private Declare Function timeGetDevCaps Lib "winmm.dll" (lpTimeCaps As TIMECAPS, ByVal uSize As Long) As Long
Private Declare Function timeBeginPeriod Lib "winmm.dll" (ByVal uPeriod As Long) As Long
Private Declare Function timeEndPeriod Lib "winmm.dll" (ByVal uPeriod As Long) As Long

Private Type TIMECAPS
   wPeriodMin As Long
   wPeriodMax As Long
End Type

Private Const TIMER_STOPPED = 0
Private Const TIMER_RUNNING = 1
Private Const PERIOD_NOT_SET = 0
Private Const TIMERR_BASE = 96
Private Const TIMERR_NOERROR = 0                ' no error
Private Const TIMERR_NOCANDO = TIMERR_BASE + 1  ' request not completed
Private Const TIMERR_STRUCT = TIMERR_BASE + 33  ' time struct size

Private glStartTime       As Long
Private glAccumulatedTime As Long
Private glRunning         As Long
Private glPeriod          As Long
Private gtcPeriods        As TIMECAPS

Private Sub Class_Initialize()
glRunning = TIMER_RUNNING ' the timer starts up running
glPeriod = PERIOD_NOT_SET
'
' Get the min & max resolution possible.
'
Call timeGetDevCaps(gtcPeriods, Len(gtcPeriods))
'
' Reset the timer to start cleanly.
'
Call Reset
End Sub

Private Sub Class_Terminate()
'
' Make sure the timer period is released.
'
If glPeriod <> PERIOD_NOT_SET Then
    Call timeEndPeriod(glPeriod)
End If
End Sub
Public Property Get ElapsedTime() As Long
'
' Return the time elapsed since the timer was last reset.
'
ElapsedTime = glAccumulatedTime - glStartTime + timeGetTime() * glRunning
End Property

Public Property Get MinPeriod() As Long
MinPeriod = gtcPeriods.wPeriodMin
End Property

Public Property Get MaxPeriod() As Long
MaxPeriod = gtcPeriods.wPeriodMax
End Property

Public Property Get Period() As Long
Period = glPeriod
End Property


Public Sub BeginPeriod(lPeriod As Long)
'
' Set the timing period.
'
Dim lReturn As Long
'
' Validate.
'
If glPeriod <> PERIOD_NOT_SET Then
    Err.Raise 51, "clsStopWatch.BeginPeriod", "Period is already set, EndPeriod call required first"
End If

With gtcPeriods
    If lPeriod < .wPeriodMin Then
        Err.Raise 51, "clsStopWatch.BeginPeriod", "Period less than minimum"
    End If
    If lPeriod > .wPeriodMax Then
        Err.Raise 51, "clsStopWatch.BeginPeriod", "Period greater than maximum"
    End If
End With
'
' Set the period.
'
lReturn = timeBeginPeriod(lPeriod)

If lReturn = TIMERR_NOERROR Then
    glPeriod = Period
Else
    glPeriod = PERIOD_NOT_SET
    Err.Raise 51, "clsStopWatch.BeginPeriod", "Error setting the multimedia timer period"
End If
End Sub
Public Sub EndPeriod(lPeriod As Long)
'
' Release timing period.
'
Dim lReturn As Long

If glPeriod = PERIOD_NOT_SET Then
    Err.Raise 51, "clsStopWatch.EndPeriod", "Period not set, BeginPeriod call required first"
End If
'
' Period must match the BeginPeriod call.
'
If lPeriod <> glPeriod Then
    Err.Raise 51, "clsStopWatch.EndPeriod", "Period does not match call to BeginPeriod"
End If

lReturn = timeEndPeriod(lPeriod)

glPeriod = PERIOD_NOT_SET

If lReturn <> TIMERR_NOERROR Then
    Err.Raise 51, "clsStopWatch.EndPeriod", "Error releasing the multimedia timer period"
End If
End Sub
Public Sub Reset()
'
' Reset the timer to zero.
'
glAccumulatedTime = 0
glStartTime = timeGetTime()
End Sub
Public Sub StartTiming()
'
' Restart the timer (after StopTiming) and accumulate time.
'
If glRunning = 0 Then
    glStartTime = timeGetTime()
    glRunning = TIMER_RUNNING
End If
End Sub
Public Sub StopTiming()
'
' Stop the timer so time doesn't accumulate.
'
If glRunning <> 0 Then
    glAccumulatedTime = glAccumulatedTime + timeGetTime() - glStartTime
    glStartTime = 0
    glRunning = TIMER_STOPPED
End If
End Sub
